"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateAggregate = void 0;
const Certificate_events_1 = require("./events/Certificate.events");
const errors_1 = require("./errors");
const ethers_1 = require("ethers");
const date_utils_1 = require("../utils/date-utils");
class CertificateAggregate {
    constructor(events) {
        this.certificate = null;
        events
            .sort((first, second) => date_utils_1.compareDates(first.createdAt, second.createdAt))
            .forEach((event) => this.apply(event));
    }
    apply(event) {
        switch (event.type) {
            case Certificate_events_1.CertificateEventType.Issued:
                this.issue(event);
                break;
            case Certificate_events_1.CertificateEventType.Transferred:
                this.transfer(event);
                break;
            case Certificate_events_1.CertificateEventType.Claimed:
                this.claim(event);
                break;
            case Certificate_events_1.CertificateEventType.ClaimPersisted:
                this.claimPersisted(event);
                break;
            case Certificate_events_1.CertificateEventType.IssuancePersisted:
                this.issuancePersisted(event);
                break;
            case Certificate_events_1.CertificateEventType.TransferPersisted:
                this.transferPersisted(event);
                break;
        }
    }
    getCertificate() {
        return this.certificate;
    }
    static fromEvents(events) {
        return new CertificateAggregate(events);
    }
    issue(event) {
        if (this.isIssued()) {
            throw new errors_1.CertificateErrors.Issuance.CertificateAlreadyIssued(event.internalCertificateId);
        }
        const payload = event.payload;
        this.certificate = {
            internalCertificateId: event.internalCertificateId,
            blockchainCertificateId: null,
            claims: [],
            creationBlockHash: '',
            creationTime: Math.floor(Date.now() / 1000),
            deviceId: payload.deviceId,
            generationStartTime: Math.floor(payload.fromTime / 1000),
            generationEndTime: Math.floor(payload.toTime / 1000),
            metadata: payload.metadata,
            owners: {
                [payload.toAddress]: payload.energyValue
            },
            claimers: {}
        };
    }
    issuancePersisted(event) {
        const { blockchainCertificateId } = event.payload;
        this.certificate.blockchainCertificateId = blockchainCertificateId;
    }
    transfer(event) {
        this.validateTransfer(event);
        const { fromAddress, toAddress, energyValue } = event.payload;
        const fromBalance = ethers_1.BigNumber.from(this.certificate.owners[fromAddress] ?? '0');
        const toBalance = ethers_1.BigNumber.from(this.certificate.owners[toAddress] ?? '0');
        const transferVolume = ethers_1.BigNumber.from(energyValue ?? fromBalance);
        const newFromBalance = fromBalance.sub(transferVolume);
        const newToBalance = toBalance.add(transferVolume);
        this.certificate.owners[fromAddress] = newFromBalance.toString();
        this.certificate.owners[toAddress] = newToBalance.toString();
    }
    transferPersisted(event) { }
    claim(event) {
        this.validateClaim(event);
        const { forAddress, energyValue, claimData } = event.payload;
        const unclaimedBalance = ethers_1.BigNumber.from(this.certificate.owners[forAddress]);
        const claimedBalance = ethers_1.BigNumber.from(this.certificate.claimers[forAddress] ? this.certificate.claimers[forAddress] : '0');
        const transferVolume = ethers_1.BigNumber.from(energyValue ?? unclaimedBalance);
        const newUnclaimedBalance = unclaimedBalance.sub(transferVolume);
        const newClaimedBalance = claimedBalance.add(transferVolume);
        this.certificate.owners[forAddress] = newUnclaimedBalance.toString();
        this.certificate.claimers[forAddress] = newClaimedBalance.toString();
        this.certificate.claims.push({
            id: event.internalCertificateId,
            from: forAddress,
            to: forAddress,
            topic: '',
            value: transferVolume.toString(),
            claimData: claimData
        });
    }
    claimPersisted(event) { }
    validateTransfer(event) {
        const { internalCertificateId, payload: { fromAddress, toAddress, energyValue } } = event;
        if (!this.isIssued()) {
            throw new errors_1.CertificateErrors.Transfer.CertificateNotIssued(internalCertificateId);
        }
        if (this.isZeroAddress(fromAddress)) {
            throw new errors_1.CertificateErrors.Transfer.FromZeroAddress(internalCertificateId);
        }
        if (this.isZeroAddress(toAddress)) {
            throw new errors_1.CertificateErrors.Transfer.ToZeroAddress(internalCertificateId);
        }
        if (!this.hasEnoughBalance(fromAddress, energyValue ?? this.certificate.owners[fromAddress])) {
            throw new errors_1.CertificateErrors.Transfer.NotEnoughBalance(internalCertificateId, fromAddress);
        }
    }
    validateClaim(event) {
        const { internalCertificateId, payload: { forAddress, energyValue } } = event;
        if (!this.isIssued()) {
            throw new errors_1.CertificateErrors.Claim.CertificateNotIssued(internalCertificateId);
        }
        if (this.isZeroAddress(forAddress)) {
            throw new errors_1.CertificateErrors.Claim.ForZeroAddress(internalCertificateId);
        }
        if (!this.hasEnoughBalance(forAddress, energyValue ?? this.certificate.owners[forAddress])) {
            throw new errors_1.CertificateErrors.Claim.NotEnoughBalance(internalCertificateId, forAddress);
        }
    }
    isIssued() {
        return this.certificate?.internalCertificateId !== undefined;
    }
    isZeroAddress(address) {
        return parseInt(address, 16) === 0;
    }
    hasEnoughBalance(address, volume) {
        const addressBalance = this.certificate?.owners[address];
        return (addressBalance !== undefined &&
            ethers_1.BigNumber.from(addressBalance).gte(ethers_1.BigNumber.from(volume)));
    }
}
exports.CertificateAggregate = CertificateAggregate;
//# sourceMappingURL=certificate.aggregate.js.map