"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Certificate_events_1 = require("../events/Certificate.events");
const testing_1 = require("@nestjs/testing");
const synchronize_strategy_1 = require("../synchronize/strategies/synchronize.strategy");
const batch_synchronize_strategy_1 = require("../synchronize/strategies/batch/batch-synchronize.strategy");
const synchronize_manager_1 = require("../synchronize/handlers/synchronize.manager");
describe('BatchSynchronizeStrategy', () => {
    let batchSynchronizeStrategy;
    const synchronizeManagerMock = {
        handleBatch: jest.fn()
    };
    const noFailedCertificateIds = {
        failedCertificateIds: []
    };
    beforeEach(async () => {
        const app = await testing_1.Test.createTestingModule({
            providers: [
                {
                    provide: synchronize_strategy_1.SYNCHRONIZE_STRATEGY,
                    useClass: batch_synchronize_strategy_1.BatchSynchronizeStrategy
                },
                {
                    provide: synchronize_manager_1.SynchronizeManager,
                    useValue: synchronizeManagerMock
                }
            ]
        }).compile();
        batchSynchronizeStrategy = app.get(synchronize_strategy_1.SYNCHRONIZE_STRATEGY);
    });
    describe('synchronize', () => {
        it('should work for no events', async () => {
            const events = [];
            await batchSynchronizeStrategy.synchronize(events);
            expect(synchronizeManagerMock.handleBatch).toBeCalledTimes(0);
        });
        it('work for new issuance event', async () => {
            const eventStub = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {}
            };
            synchronizeManagerMock.handleBatch.mockResolvedValueOnce({ failedCertificateIds: [] });
            const events = [eventStub];
            await batchSynchronizeStrategy.synchronize(events);
            expect(synchronizeManagerMock.handleBatch).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handleBatch).toBeCalledWith(events);
        });
        it('work for new claim event', async () => {
            const eventStub = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Claimed,
                commandId: 1,
                payload: {}
            };
            const events = [eventStub];
            synchronizeManagerMock.handleBatch.mockResolvedValueOnce({ failedCertificateIds: [] });
            await batchSynchronizeStrategy.synchronize(events);
            expect(synchronizeManagerMock.handleBatch).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handleBatch).toBeCalledWith(events);
        });
        it('work for new transfer event', async () => {
            const eventStub = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Transferred,
                commandId: 1,
                payload: {}
            };
            const events = [eventStub];
            synchronizeManagerMock.handleBatch.mockResolvedValueOnce({ failedCertificateIds: [] });
            await batchSynchronizeStrategy.synchronize(events);
            expect(synchronizeManagerMock.handleBatch).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handleBatch).toBeCalledWith(events);
        });
        it('should not be processed further if previous step failed for that certificate', async () => {
            const claimedEvent = {
                internalCertificateId: 0,
                id: 0,
                type: Certificate_events_1.CertificateEventType.Claimed,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const transferredEvent = {
                internalCertificateId: 0,
                id: 0,
                type: Certificate_events_1.CertificateEventType.Transferred,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const events = [claimedEvent, transferredEvent];
            synchronizeManagerMock.handleBatch.mockResolvedValueOnce({
                failedCertificateIds: [claimedEvent.internalCertificateId]
            });
            await batchSynchronizeStrategy.synchronize(events);
            expect(synchronizeManagerMock.handleBatch).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handleBatch).toBeCalledWith([claimedEvent]);
        });
        it('should be processed further if previous step succeeded  for that certificate', async () => {
            const claimedEvent = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Claimed,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const transferredEvent = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Transferred,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const events = [claimedEvent, transferredEvent];
            synchronizeManagerMock.handleBatch
                .mockResolvedValueOnce(noFailedCertificateIds)
                .mockResolvedValueOnce(noFailedCertificateIds);
            await batchSynchronizeStrategy.synchronize(events);
            expect(synchronizeManagerMock.handleBatch).toBeCalledTimes(2);
            expect(synchronizeManagerMock.handleBatch).toHaveBeenNthCalledWith(1, [claimedEvent]);
            expect(synchronizeManagerMock.handleBatch).toHaveBeenNthCalledWith(2, [
                transferredEvent
            ]);
        });
    });
});
//# sourceMappingURL=batch-synchronize.spec.js.map