"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const testing_1 = require("@nestjs/testing");
const repository_keys_1 = require("../repositories/repository.keys");
const claim_persist_handler_1 = require("../synchronize/handlers/claim-persist.handler");
const Certificate_events_1 = require("../events/Certificate.events");
const batch_configuration_1 = require("../synchronize/strategies/batch/batch.configuration");
const __1 = require("../..");
const offchain_certificate_service_1 = require("../offchain-certificate.service");
describe('ClaimPersistHandler', () => {
    let claimPersistHandler;
    const certEventRepoMock = {};
    const offchainCertificateServiceMock = {
        persistError: jest.fn(),
        claimPersisted: jest.fn()
    };
    const certificateServiceMock = {
        claim: jest.fn(),
        batchClaim: jest.fn()
    };
    beforeEach(async () => {
        const app = await testing_1.Test.createTestingModule({
            providers: [
                {
                    provide: __1.ONCHAIN_CERTIFICATE_SERVICE_TOKEN,
                    useValue: certificateServiceMock
                },
                {
                    provide: offchain_certificate_service_1.OffChainCertificateService,
                    useValue: offchainCertificateServiceMock
                },
                {
                    provide: repository_keys_1.CERTIFICATE_EVENT_REPOSITORY,
                    useValue: certEventRepoMock
                },
                {
                    provide: batch_configuration_1.BATCH_CONFIGURATION_TOKEN,
                    useValue: batch_configuration_1.batchConfiguration
                },
                claim_persist_handler_1.ClaimPersistHandler
            ]
        }).compile();
        claimPersistHandler = app.get(claim_persist_handler_1.ClaimPersistHandler);
    });
    const createEvent = (values = {}) => ({
        type: Certificate_events_1.CertificateEventType.Claimed,
        commandId: 0,
        internalCertificateId: 0,
        id: 0,
        createdAt: new Date(),
        payload: {},
        version: 0,
        ...values
    });
    describe('#canHandle', () => {
        it('should return true for claimed event type', () => {
            let event = createEvent({ type: Certificate_events_1.CertificateEventType.Claimed });
            const canHandle = claimPersistHandler.canHandle(event);
            expect(canHandle).toEqual(true);
        });
        it.each(Object.values(Certificate_events_1.CertificateEventType)
            .filter((type) => type !== Certificate_events_1.CertificateEventType.Claimed)
            .map((eventType) => [eventType]))('should return false for %s event type', (eventType) => {
            const event = createEvent({ type: eventType });
            const canHandle = claimPersistHandler.canHandle(event);
            expect(canHandle).toEqual(false);
        });
    });
    describe('#handle', () => {
        it('should trigger claim persisted if claim was successful', async () => {
            const event = createEvent();
            certificateServiceMock.claim.mockResolvedValueOnce({ success: true });
            await claimPersistHandler.handle(event);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(0);
            expect(certificateServiceMock.claim).toBeCalledTimes(1);
            expect(certificateServiceMock.claim).toBeCalledWith(event.payload);
            expect(offchainCertificateServiceMock.claimPersisted).toBeCalledTimes(1);
            expect(offchainCertificateServiceMock.claimPersisted).toBeCalledWith(event.internalCertificateId, {});
        });
        it('should report persist error if synchronization failed', async () => {
            const event = createEvent();
            certificateServiceMock.claim.mockResolvedValueOnce({ success: false });
            await claimPersistHandler.handle(event);
            expect(certificateServiceMock.claim).toBeCalledTimes(1);
            expect(certificateServiceMock.claim).toBeCalledWith(event.payload);
            expect(offchainCertificateServiceMock.claimPersisted).toBeCalledTimes(0);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(1);
            expect(offchainCertificateServiceMock.persistError).toBeCalledWith(event.internalCertificateId, { errorMessage: expect.any(String) });
        });
    });
    describe('#handleBatch', () => {
        it('should trigger claim persisted for all events if batchClaim was successful', async () => {
            const event = createEvent();
            certificateServiceMock.batchClaim.mockResolvedValueOnce({ success: true });
            await claimPersistHandler.handleBatch([event, event]);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(0);
            expect(certificateServiceMock.batchClaim).toBeCalledTimes(1);
            expect(certificateServiceMock.batchClaim).toBeCalledWith([
                event.payload,
                event.payload
            ]);
            expect(offchainCertificateServiceMock.claimPersisted).toBeCalledTimes(2);
            expect(offchainCertificateServiceMock.claimPersisted).nthCalledWith(1, event.internalCertificateId, {});
            expect(offchainCertificateServiceMock.claimPersisted).nthCalledWith(2, event.internalCertificateId, {});
        });
        it('should report persist error for all events if synchronization failed', async () => {
            const event = createEvent();
            certificateServiceMock.batchClaim.mockResolvedValueOnce({ success: false });
            await claimPersistHandler.handleBatch([event, event]);
            expect(offchainCertificateServiceMock.claimPersisted).toBeCalledTimes(0);
            expect(certificateServiceMock.batchClaim).toBeCalledTimes(1);
            expect(certificateServiceMock.batchClaim).toBeCalledWith([
                event.payload,
                event.payload
            ]);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(2);
            expect(offchainCertificateServiceMock.persistError).nthCalledWith(1, event.internalCertificateId, { errorMessage: expect.any(String) });
            expect(offchainCertificateServiceMock.persistError).nthCalledWith(2, event.internalCertificateId, { errorMessage: expect.any(String) });
        });
    });
});
//# sourceMappingURL=claim-persist.handler.spec.js.map