"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Certificate_events_1 = require("../events/Certificate.events");
const events_batch_queue_1 = require("../synchronize/strategies/batch/events.batch-queue");
describe('EventsBatchQueue', () => {
    describe('#isEmpty', () => {
        it('should return true if no events are on the queue', async () => {
            const events = [];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            expect(eventsBatchQueue.isEmpty()).toEqual(true);
        });
        it('should return false if there is one event on queue', async () => {
            const eventStub = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {}
            };
            const events = [eventStub];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            expect(eventsBatchQueue.isEmpty()).toEqual(false);
        });
        it('should return false if there are two events on queue', async () => {
            const eventStub = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const otherEventStup = {
                id: 0,
                internalCertificateId: 1,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const events = [eventStub, otherEventStup];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            expect(eventsBatchQueue.isEmpty()).toEqual(false);
        });
    });
    describe('#popBatch', () => {
        it('should return empty array if there are no events', async () => {
            const events = [];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            expect(eventsBatchQueue.popBatch()).toEqual([]);
        });
        it('should return empty array if there are no events after processing everything', async () => {
            const eventStub = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date(1)
            };
            const events = [eventStub];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            expect(eventsBatchQueue.popBatch()).toEqual([eventStub]);
            expect(eventsBatchQueue.popBatch()).toEqual([]);
        });
        it('should return all events if they are for different certificates', async () => {
            const eventStub = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const otherEventStup = {
                id: 0,
                internalCertificateId: 1,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const events = [eventStub, otherEventStup];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            expect(eventsBatchQueue.popBatch()).toEqual([eventStub, otherEventStup]);
        });
        it('should return only first event if they are for the same certificate', async () => {
            const eventStub = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date(1)
            };
            const earlierEvent = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date(0)
            };
            const events = [eventStub, earlierEvent];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            expect(eventsBatchQueue.popBatch()).toEqual([earlierEvent]);
        });
    });
    describe('#removeEventsForCertificateIds', () => {
        it('should return remove all events if they are for the same certificate', async () => {
            let internalCertificateId = 0;
            const eventStub = {
                id: 0,
                internalCertificateId,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const otherEventStup = {
                id: 0,
                internalCertificateId,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date()
            };
            const events = [eventStub, otherEventStup];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            eventsBatchQueue.removeEventsForCertificateIds([internalCertificateId]);
            expect(eventsBatchQueue.isEmpty()).toEqual(true);
        });
        it('should remove only the events for specific certificate', async () => {
            const eventStub = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date(1)
            };
            const eventForDifferentCertificate = {
                id: 0,
                internalCertificateId: 1,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                payload: {},
                createdAt: new Date(0)
            };
            const events = [eventStub, eventForDifferentCertificate];
            const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
            eventsBatchQueue.removeEventsForCertificateIds([0]);
            expect(eventsBatchQueue.popBatch()).toEqual([eventForDifferentCertificate]);
        });
    });
});
//# sourceMappingURL=events.batch-queue.spec.js.map