"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const testing_1 = require("@nestjs/testing");
const repository_keys_1 = require("../repositories/repository.keys");
const Certificate_events_1 = require("../events/Certificate.events");
const issue_persist_handler_1 = require("../synchronize/handlers/issue-persist.handler");
const batch_configuration_1 = require("../synchronize/strategies/batch/batch.configuration");
const __1 = require("../..");
const offchain_certificate_service_1 = require("../offchain-certificate.service");
describe('IssuePersistHandler', () => {
    let issuePersistHandler;
    const certEventRepoMock = {};
    const offchainCertificateServiceMock = {
        persistError: jest.fn(),
        issuePersisted: jest.fn()
    };
    const certificateServiceMock = {
        issue: jest.fn(),
        batchIssue: jest.fn()
    };
    beforeEach(async () => {
        const app = await testing_1.Test.createTestingModule({
            providers: [
                {
                    provide: __1.ONCHAIN_CERTIFICATE_SERVICE_TOKEN,
                    useValue: certificateServiceMock
                },
                {
                    provide: offchain_certificate_service_1.OffChainCertificateService,
                    useValue: offchainCertificateServiceMock
                },
                {
                    provide: repository_keys_1.CERTIFICATE_EVENT_REPOSITORY,
                    useValue: certEventRepoMock
                },
                {
                    provide: batch_configuration_1.BATCH_CONFIGURATION_TOKEN,
                    useValue: batch_configuration_1.batchConfiguration
                },
                issue_persist_handler_1.IssuePersistHandler
            ]
        }).compile();
        issuePersistHandler = app.get(issue_persist_handler_1.IssuePersistHandler);
    });
    const createEvent = (values = {}) => ({
        type: Certificate_events_1.CertificateEventType.Issued,
        commandId: 0,
        internalCertificateId: 0,
        id: 0,
        createdAt: new Date(),
        payload: {
            fromTime: new Date().getTime(),
            toTime: new Date().getTime()
        },
        version: 0,
        ...values
    });
    describe('#canHandle', () => {
        it('should return true for issued event type', () => {
            let event = createEvent({ type: Certificate_events_1.CertificateEventType.Issued });
            const canHandle = issuePersistHandler.canHandle(event);
            expect(canHandle).toEqual(true);
        });
        it.each(Object.values(Certificate_events_1.CertificateEventType)
            .filter((type) => type !== Certificate_events_1.CertificateEventType.Issued)
            .map((eventType) => [eventType]))('should return false for %s event type', (eventType) => {
            const event = createEvent({ type: eventType });
            const canHandle = issuePersistHandler.canHandle(event);
            expect(canHandle).toEqual(false);
        });
    });
    describe('#handle', () => {
        it('should trigger issue persisted if issue was successful', async () => {
            const event = createEvent();
            const blockchainCertificateId = 0;
            certificateServiceMock.issue.mockResolvedValueOnce({ id: blockchainCertificateId });
            await issuePersistHandler.handle(event);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(0);
            expect(certificateServiceMock.issue).toBeCalledTimes(1);
            expect(certificateServiceMock.issue).toBeCalledWith({
                ...event.payload,
                fromTime: new Date(event.payload.fromTime),
                toTime: new Date(event.payload.toTime)
            });
            expect(offchainCertificateServiceMock.issuePersisted).toBeCalledTimes(1);
            expect(offchainCertificateServiceMock.issuePersisted).toBeCalledWith(event.internalCertificateId, {
                blockchainCertificateId
            });
        });
        it('should report persist error if synchronization failed', async () => {
            const event = createEvent();
            const eventPayload = event.payload;
            certificateServiceMock.issue.mockResolvedValueOnce(undefined);
            await issuePersistHandler.handle(event);
            expect(certificateServiceMock.issue).toBeCalledTimes(1);
            expect(certificateServiceMock.issue).toBeCalledWith({
                ...eventPayload,
                fromTime: new Date(eventPayload.fromTime),
                toTime: new Date(eventPayload.toTime)
            });
            expect(offchainCertificateServiceMock.issuePersisted).toBeCalledTimes(0);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(1);
            expect(offchainCertificateServiceMock.persistError).toBeCalledWith(event.internalCertificateId, { errorMessage: expect.any(String) });
        });
    });
    describe('#handleBatch', () => {
        it('should trigger issue persisted for all events if batchIssue was successful', async () => {
            const event = createEvent();
            const eventPayload = event.payload;
            const expectedEventPayloadCall = {
                ...eventPayload,
                fromTime: new Date(eventPayload.fromTime),
                toTime: new Date(eventPayload.toTime)
            };
            const resolvedCertificateIds = [0, 1];
            certificateServiceMock.batchIssue.mockResolvedValueOnce(resolvedCertificateIds);
            await issuePersistHandler.handleBatch([event, event]);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(0);
            expect(certificateServiceMock.batchIssue).toBeCalledTimes(1);
            expect(certificateServiceMock.batchIssue).toBeCalledWith([
                expectedEventPayloadCall,
                expectedEventPayloadCall
            ]);
            expect(offchainCertificateServiceMock.issuePersisted).toBeCalledTimes(2);
            expect(offchainCertificateServiceMock.issuePersisted).nthCalledWith(1, event.internalCertificateId, {
                blockchainCertificateId: resolvedCertificateIds[0]
            });
            expect(offchainCertificateServiceMock.issuePersisted).nthCalledWith(2, event.internalCertificateId, {
                blockchainCertificateId: resolvedCertificateIds[1]
            });
        });
        it('should report persist error for all events if synchronization failed', async () => {
            const event = createEvent();
            certificateServiceMock.batchIssue.mockResolvedValueOnce(undefined);
            const eventPayload = event.payload;
            const expectedEventPayloadCall = {
                ...eventPayload,
                fromTime: new Date(eventPayload.fromTime),
                toTime: new Date(eventPayload.toTime)
            };
            await issuePersistHandler.handleBatch([event, event]);
            expect(offchainCertificateServiceMock.issuePersisted).toBeCalledTimes(0);
            expect(certificateServiceMock.batchIssue).toBeCalledTimes(1);
            expect(certificateServiceMock.batchIssue).toBeCalledWith([
                expectedEventPayloadCall,
                expectedEventPayloadCall
            ]);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(2);
            expect(offchainCertificateServiceMock.persistError).nthCalledWith(1, event.internalCertificateId, { errorMessage: expect.any(String) });
            expect(offchainCertificateServiceMock.persistError).nthCalledWith(2, event.internalCertificateId, { errorMessage: expect.any(String) });
        });
    });
});
//# sourceMappingURL=issue-persist.handler.spec.js.map