"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Certificate_events_1 = require("../events/Certificate.events");
const testing_1 = require("@nestjs/testing");
const synchronize_strategy_1 = require("../synchronize/strategies/synchronize.strategy");
const serial_synchronize_strategy_1 = require("../synchronize/strategies/serial/serial-synchronize.strategy");
const synchronize_manager_1 = require("../synchronize/handlers/synchronize.manager");
const batch_configuration_1 = require("../synchronize/strategies/batch/batch.configuration");
describe('BlockchainSynchronize', () => {
    let serialStrategy;
    const synchronizeManagerMock = {
        handle: jest.fn()
    };
    beforeEach(async () => {
        const app = await testing_1.Test.createTestingModule({
            providers: [
                {
                    provide: synchronize_strategy_1.SYNCHRONIZE_STRATEGY,
                    useClass: serial_synchronize_strategy_1.SerialSynchronizeStrategy
                },
                {
                    provide: synchronize_manager_1.SynchronizeManager,
                    useValue: synchronizeManagerMock
                },
                {
                    provide: batch_configuration_1.BATCH_CONFIGURATION_TOKEN,
                    useValue: batch_configuration_1.batchConfiguration
                }
            ]
        }).compile();
        serialStrategy = app.get(synchronize_strategy_1.SYNCHRONIZE_STRATEGY);
    });
    describe('synchronize', () => {
        it('should work for no events', async () => {
            const events = [];
            await serialStrategy.synchronize(events);
            expect(synchronizeManagerMock.handle).toBeCalledTimes(0);
        });
        it('work for new issuance event', async () => {
            const eventStub = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                createdAt: new Date(),
                payload: {}
            };
            const events = [eventStub];
            synchronizeManagerMock.handle.mockResolvedValueOnce({ success: true });
            await serialStrategy.synchronize(events);
            expect(synchronizeManagerMock.handle).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handle).toBeCalledWith(eventStub);
        });
        it('work for new claim event', async () => {
            const eventStub = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Claimed,
                commandId: 1,
                createdAt: new Date(),
                payload: {}
            };
            const events = [eventStub];
            synchronizeManagerMock.handle.mockResolvedValueOnce({ success: true });
            await serialStrategy.synchronize(events);
            expect(synchronizeManagerMock.handle).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handle).toBeCalledWith(eventStub);
        });
        it('work for new transfer event', async () => {
            const eventStub = {
                id: 0,
                type: Certificate_events_1.CertificateEventType.Transferred,
                commandId: 1,
                createdAt: new Date(),
                payload: {}
            };
            synchronizeManagerMock.handle.mockResolvedValueOnce({ success: true });
            const events = [eventStub];
            await serialStrategy.synchronize(events);
            expect(synchronizeManagerMock.handle).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handle).toBeCalledWith(eventStub);
        });
        it('should not synchronize event if it failed for that certificate ids', async () => {
            const issueEvent = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Issued,
                commandId: 1,
                createdAt: new Date(0),
                payload: {}
            };
            const claimEvent = {
                id: 0,
                internalCertificateId: 0,
                type: Certificate_events_1.CertificateEventType.Claimed,
                commandId: 1,
                createdAt: new Date(1),
                payload: {}
            };
            const events = [issueEvent, claimEvent];
            synchronizeManagerMock.handle.mockResolvedValueOnce({ success: false });
            await serialStrategy.synchronize(events);
            expect(synchronizeManagerMock.handle).toBeCalledTimes(1);
            expect(synchronizeManagerMock.handle).toBeCalledWith(issueEvent);
        });
    });
});
//# sourceMappingURL=serial-synchronize.spec.js.map