"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const testing_1 = require("@nestjs/testing");
const repository_keys_1 = require("../repositories/repository.keys");
const transfer_persist_handler_1 = require("../synchronize/handlers/transfer-persist.handler");
const Certificate_events_1 = require("../events/Certificate.events");
const batch_configuration_1 = require("../synchronize/strategies/batch/batch.configuration");
const __1 = require("../..");
const offchain_certificate_service_1 = require("../offchain-certificate.service");
describe('TransferPersistHandler', () => {
    let transferPersistHandler;
    const certEventRepoMock = {};
    const offchainCertificateServiceMock = {
        persistError: jest.fn(),
        transferPersisted: jest.fn()
    };
    const certificateServiceMock = {
        transfer: jest.fn(),
        batchTransfer: jest.fn()
    };
    beforeEach(async () => {
        const app = await testing_1.Test.createTestingModule({
            providers: [
                {
                    provide: __1.ONCHAIN_CERTIFICATE_SERVICE_TOKEN,
                    useValue: certificateServiceMock
                },
                {
                    provide: offchain_certificate_service_1.OffChainCertificateService,
                    useValue: offchainCertificateServiceMock
                },
                {
                    provide: repository_keys_1.CERTIFICATE_EVENT_REPOSITORY,
                    useValue: certEventRepoMock
                },
                {
                    provide: batch_configuration_1.BATCH_CONFIGURATION_TOKEN,
                    useValue: batch_configuration_1.batchConfiguration
                },
                transfer_persist_handler_1.TransferPersistHandler
            ]
        }).compile();
        transferPersistHandler = app.get(transfer_persist_handler_1.TransferPersistHandler);
    });
    const createEvent = (values = {}) => ({
        type: Certificate_events_1.CertificateEventType.Transferred,
        commandId: 0,
        internalCertificateId: 0,
        id: 0,
        createdAt: new Date(),
        payload: {},
        version: 0,
        ...values
    });
    describe('#canHandle', () => {
        it('should return true for transfered event type', () => {
            let event = createEvent({ type: Certificate_events_1.CertificateEventType.Transferred });
            const canHandle = transferPersistHandler.canHandle(event);
            expect(canHandle).toEqual(true);
        });
        it.each(Object.values(Certificate_events_1.CertificateEventType)
            .filter((type) => type !== Certificate_events_1.CertificateEventType.Transferred)
            .map((eventType) => [eventType]))('should return false for %s event type', (eventType) => {
            const event = createEvent({ type: eventType });
            const canHandle = transferPersistHandler.canHandle(event);
            expect(canHandle).toEqual(false);
        });
    });
    describe('#handle', () => {
        it('should trigger transfer persisted if transfer was successful', async () => {
            const event = createEvent();
            certificateServiceMock.transfer.mockResolvedValueOnce({ success: true });
            await transferPersistHandler.handle(event);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(0);
            expect(certificateServiceMock.transfer).toBeCalledTimes(1);
            expect(certificateServiceMock.transfer).toBeCalledWith(event.payload);
            expect(offchainCertificateServiceMock.transferPersisted).toBeCalledTimes(1);
            expect(offchainCertificateServiceMock.transferPersisted).toBeCalledWith(event.internalCertificateId, {});
        });
        it('should report persist error if synchronization failed', async () => {
            const event = createEvent();
            certificateServiceMock.transfer.mockResolvedValueOnce({ success: false });
            await transferPersistHandler.handle(event);
            expect(certificateServiceMock.transfer).toBeCalledTimes(1);
            expect(certificateServiceMock.transfer).toBeCalledWith(event.payload);
            expect(offchainCertificateServiceMock.transferPersisted).toBeCalledTimes(0);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(1);
            expect(offchainCertificateServiceMock.persistError).toBeCalledWith(event.internalCertificateId, { errorMessage: expect.any(String) });
        });
    });
    describe('#handleBatch', () => {
        it('should trigger transfer persisted for all events if batchTransfer was successful', async () => {
            const event = createEvent();
            certificateServiceMock.batchTransfer.mockResolvedValueOnce({ success: true });
            await transferPersistHandler.handleBatch([event, event]);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(0);
            expect(certificateServiceMock.batchTransfer).toBeCalledTimes(1);
            expect(certificateServiceMock.batchTransfer).toBeCalledWith([
                event.payload,
                event.payload
            ]);
            expect(offchainCertificateServiceMock.transferPersisted).toBeCalledTimes(2);
            expect(offchainCertificateServiceMock.transferPersisted).nthCalledWith(1, event.internalCertificateId, {});
            expect(offchainCertificateServiceMock.transferPersisted).nthCalledWith(2, event.internalCertificateId, {});
        });
        it('should report persist error for all events if synchronization failed', async () => {
            const event = createEvent();
            certificateServiceMock.batchTransfer.mockResolvedValueOnce({ success: false });
            await transferPersistHandler.handleBatch([event, event]);
            expect(offchainCertificateServiceMock.transferPersisted).toBeCalledTimes(0);
            expect(certificateServiceMock.batchTransfer).toBeCalledTimes(1);
            expect(certificateServiceMock.batchTransfer).toBeCalledWith([
                event.payload,
                event.payload
            ]);
            expect(offchainCertificateServiceMock.persistError).toBeCalledTimes(2);
            expect(offchainCertificateServiceMock.persistError).nthCalledWith(1, event.internalCertificateId, { errorMessage: expect.any(String) });
            expect(offchainCertificateServiceMock.persistError).nthCalledWith(2, event.internalCertificateId, { errorMessage: expect.any(String) });
        });
    });
});
//# sourceMappingURL=transfer-persist.handler.spec.js.map